using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

namespace WindowsApplication1 {
	/// <summary>
	/// A simple example to show how a timer can be used to animated a change 
	/// in an object property such as movement.  Even though no threads are used,
	/// the animation still responds to real-time, meaning that on a fast computer,
	/// there are more inbetween frames.  This example also uses slow-in-slow-out
	/// animation to make the movement feel more realistic.
	/// 
	/// Ben Bederson
	/// University of Maryland
	/// May 2005
	/// </summary>
	public class AnimatedMovement : System.Windows.Forms.Form {
		private System.ComponentModel.Container components = null;

		private Point currentPoint = new Point(0, 0);
		private Point fromPoint = new Point(0, 0);
		private Point toPoint = new Point(200, 100);
		private Font font = new Font("Arial", 20);
	
		private DateTime startTime;
		private int duration = 1000;
		private int numRenders;

		public AnimatedMovement() {
			SetStyle(ControlStyles.DoubleBuffer, true);
			SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			SetStyle(ControlStyles.UserPaint, true);

			InitializeComponent();
			MouseDown += new MouseEventHandler(AnimatedMovement_MouseDown);

			StartMovement();
		}

		private void StartMovement() {
			numRenders = 0;
			startTime = DateTime.Now;
			Invalidate();
		}

		protected override void OnPaint(PaintEventArgs e) {
			base.OnPaint (e);

			bool done = UpdatePosition();
			// First paint the scene with the currently computed properties
			e.Graphics.FillRectangle(Brushes.Blue, currentPoint.X, currentPoint.Y, 200, 200);
			if (!done) {
				Invalidate();
			}

			// Debugging code to display frames per second in display
			DateTime now = DateTime.Now;
			TimeSpan elapsedSpan = now.Subtract(startTime);
			float elapsed = (elapsedSpan.Seconds * 1000.0f + elapsedSpan.Milliseconds) / 1000.0f;
			numRenders++;
			int fps = (int)(numRenders / elapsed);
			e.Graphics.DrawString(fps.ToString(), font, Brushes.Yellow, 50, 50);
			// Uncomment the next line to force rendering to be slow
			// System.Threading.Thread.Sleep(200);
		}

		// Update the scene based on elapsed time, and request a new render if not complete
		private bool UpdatePosition() {
			DateTime now = DateTime.Now;
			TimeSpan elapsed = now.Subtract(startTime);
			float t = (float)((elapsed.Seconds * 1000) + elapsed.Milliseconds) / duration;
			if (t > 1) {
				t = 1;
			}

			float t1 = SISO(t);
			int x = (int)LERP(t1, fromPoint.X, toPoint.X);
			int y = (int)LERP(t1, fromPoint.Y, toPoint.Y);
			currentPoint = new Point(x, y);

			return (t >= 1);
		}

		// Linearly interpolate from p1 to p2
		private float LERP(float t, float p1, float p2) {
			return p1 + t * (p2 - p1);
		}

		// Warp t, in [0, 1], to a slow-in-slow-out version of t
		private float SISO(float t) {
			if (t < 0.5) {
				return 2.0f * t * t;
			} else {
				float complement = 1.0f - t;
				return 1.0f - (2.0f * complement * complement);
			}
		}

		private void AnimatedMovement_MouseDown(object sender, MouseEventArgs e) {
			StartMovement();
		}

		#region Dispose
		protected override void Dispose( bool disposing ) {
			if( disposing ) {
				if (components != null) {
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}
		#endregion

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent() {
			// 
			// AnimatedMovement
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(544, 438);
			this.Name = "AnimatedMovement";
			this.Text = "Form1";

		}
		#endregion

		[STAThread]
		static void Main() {
			Application.Run(new AnimatedMovement());
		}
	}
}